@@ -501,7 +501,6 @@ class VariableWindowAroundChunks(AbstractCoordProducer):
         self.verbose = verbose
         self.plot_save_dir = plot_save_dir
 
-
     @classmethod
     def from_hdf5(cls, grp):
         sliding = np.array(grp["sliding"]).astype("int")
@@ -631,7 +630,7 @@ class VariableWindowAroundChunks(AbstractCoordProducer):
                 val_transformer=precision_transformer) 
 
         else:
-            precision_transformer = tnt_results.precision_transformer
+            precision_transformer = tnt_results.val_transformer
             (precisiontransformed_score_track,
              precisiontransformed_bestwindowsizeidxs) =\
                 precision_transformer.transform_score_track(
