@@ -6,16 +6,17 @@ from .. import aggregator
 from .. import core
 from .. import util
 from .. import seqlet_embedding
+from .. import pattern_filterer as pattern_filterer_module
+from joblib import Parallel, delayed
 from collections import defaultdict, OrderedDict, Counter
-from scipy.sparse import coo_matrix
 import numpy as np
 import time
 import sys
 import gc
+import json
 from ..util import print_memory_use
 
 
-
 def get_seqlet_neighbors_with_initcluster(
         nearest_neighbors_to_compute,
         coarse_affmat, initclusters): 
@@ -87,7 +88,6 @@ class TfModiscoSeqletsToPatternsFactory(object):
                         seqlet_embedding.advanced_gapped_kmer
                                         .AdvancedGappedKmerEmbedderFactory()),
 
-                       nn_n_jobs=None,
                        nearest_neighbors_to_compute=500,
 
                        affmat_correlation_threshold=0.15,
@@ -116,14 +116,17 @@ class TfModiscoSeqletsToPatternsFactory(object):
                        prob_and_pertrack_sim_dealbreaker_thresholds=[
                         (0.4, 0.75), (0.2,0.8), (0.1, 0.85), (0.0,0.9)],
 
+                       subcluster_perplexity=50,
                        merging_max_seqlets_subsample=300,
-                       threshold_for_spurious_merge_detection=0.8,
+                       #threshold_for_spurious_merge_detection=0.8,
 
-                       min_similarity_for_seqlet_assignment=0.2,
+                       #min_similarity_for_seqlet_assignment=0.2,
                        final_min_cluster_size=30,
+                       min_ic_in_window=0.6,#total IC in some windowsize window 
+                       min_ic_windowsize=6,
+                       ppm_pseudocount=0.001,
 
-                       final_flank_to_add=10,
-                       final_subcluster_perplexity=30,
+                       final_flank_to_add=0,
 
                        verbose=True, seed=1234):
 
@@ -139,7 +142,6 @@ class TfModiscoSeqletsToPatternsFactory(object):
 
         self.embedder_factory = embedder_factory
 
-        self.nn_n_jobs = (nn_n_jobs if nn_n_jobs is not None else n_cores)
         self.nearest_neighbors_to_compute = nearest_neighbors_to_compute
 
         self.affmat_correlation_threshold = affmat_correlation_threshold
@@ -166,6 +168,9 @@ class TfModiscoSeqletsToPatternsFactory(object):
         self.trim_to_window_size = trim_to_window_size
         self.initial_flank_to_add = initial_flank_to_add 
 
+        #subclustering
+        self.subcluster_perplexity=subcluster_perplexity
+
         #merging similar patterns
         self.prob_and_pertrack_sim_merge_thresholds =\
             prob_and_pertrack_sim_merge_thresholds
@@ -173,17 +178,19 @@ class TfModiscoSeqletsToPatternsFactory(object):
             prob_and_pertrack_sim_dealbreaker_thresholds
 
         self.merging_max_seqlets_subsample = merging_max_seqlets_subsample
-        self.threshold_for_spurious_merge_detection =\
-            threshold_for_spurious_merge_detection
+        #self.threshold_for_spurious_merge_detection =\
+        #    threshold_for_spurious_merge_detection
 
         #reassignment settings
-        self.min_similarity_for_seqlet_assignment =\
-            min_similarity_for_seqlet_assignment
+        #self.min_similarity_for_seqlet_assignment =\
+        #    min_similarity_for_seqlet_assignment
         self.final_min_cluster_size = final_min_cluster_size
+        self.min_ic_in_window = min_ic_in_window
+        self.min_ic_windowsize = min_ic_windowsize
+        self.ppm_pseudocount = ppm_pseudocount
 
         #final postprocessor settings
         self.final_flank_to_add=final_flank_to_add
-        self.final_subcluster_perplexity=final_subcluster_perplexity
 
         #other settings
         self.verbose = verbose
@@ -199,7 +206,6 @@ class TfModiscoSeqletsToPatternsFactory(object):
                 ('embedder_factory',
                  self.embedder_factory.get_jsonable_config()),
                 ('num_mismatches', self.num_mismatches),
-                ('nn_n_jobs', self.nn_n_jobs),
                 ('nearest_neighbors_to_compute',
                  self.nearest_neighbors_to_compute),
                 ('affmat_correlation_threshold',
@@ -221,20 +227,22 @@ class TfModiscoSeqletsToPatternsFactory(object):
                 ('min_num_to_trim_to', self.min_num_to_trim_to),
                 ('trim_to_window_size', self.trim_to_window_size),
                 ('initial_flank_to_add', self.initial_flank_to_add),
+                ('subcluster_perplexity', self.subcluster_perplexity),
                 ('prob_and_pertrack_sim_merge_thresholds',
                  self.prob_and_pertrack_sim_merge_thresholds),
                 ('prob_and_pertrack_sim_dealbreaker_thresholds',
                  self.prob_and_pertrack_sim_dealbreaker_thresholds),
                 ('merging_max_seqlets_subsample',
                  self.merging_max_seqlets_subsample),
-                ('threshold_for_spurious_merge_detection',
-                 self.threshold_for_spurious_merge_detection),
+                #('threshold_for_spurious_merge_detection',
+                # self.threshold_for_spurious_merge_detection),
                 ('min_similarity_for_seqlet_assignment',
                  self.min_similarity_for_seqlet_assignment),
                 ('final_min_cluster_size', self.final_min_cluster_size),
+                ('min_ic_in_window', self.min_ic_in_window),
+                ('min_ic_windowsize', self.min_ic_windowsize),
+                ('ppm_pseudocount', self.ppm_pseudocount),
                 ('final_flank_to_add', self.final_flank_to_add),
-                ('final_subcluster_perplexity',
-                 self.final_subcluster_perplexity)
                 ]) 
         return to_return
 
@@ -247,7 +255,7 @@ class TfModiscoSeqletsToPatternsFactory(object):
         bg_freq = np.mean(
             track_set.track_name_to_data_track[onehot_track_name].fwd_tracks,
             axis=(0,1))
-        assert len(bg_freq.shape)==1
+        assert len(bg_freq.shape)==1 
 
         assert len(track_signs)==len(hypothetical_contribs_track_names)
         assert len(track_signs)==len(contrib_scores_track_names)
@@ -286,7 +294,6 @@ class TfModiscoSeqletsToPatternsFactory(object):
                 sparse_affmat_from_fwdnrev1dvecs=\
                     affinitymat.core.SparseNumpyCosineSimFromFwdAndRevOneDVecs(
                         n_neighbors=self.nearest_neighbors_to_compute, 
-                        nn_n_jobs=self.nn_n_jobs, 
                         verbose=self.verbose),
                 verbose=self.verbose)
 
@@ -305,14 +312,18 @@ class TfModiscoSeqletsToPatternsFactory(object):
                 verbose=self.verbose)
 
         aff_to_dist_mat = affinitymat.transformers.AffToDistViaInvLogistic() 
-        density_adapted_affmat_transformer =\
-            affinitymat.transformers.NNTsneConditionalProbs(
-                perplexity=self.tsne_perplexity,
-                aff_to_dist_mat=aff_to_dist_mat)
+        #density_adapted_affmat_transformer =\
+        #    affinitymat.transformers.NNTsneConditionalProbs(
+        #        perplexity=self.tsne_perplexity,
+        #        aff_to_dist_mat=aff_to_dist_mat)
 
         #prepare the clusterers for the different rounds
-        affmat_transformer_r1 = affinitymat.transformers.SymmetrizeByAddition(
-                                probability_normalize=True)
+        # No longer a need for symmetrization because am symmetrizing by
+        # taking the geometric mean elsewhere
+        affmat_transformer_r1 =\
+            affinitymat.transformers.AdhocAffMatTransformer(lambda x: x)
+        #affinitymat.transformers.SymmetrizeByAddition(
+        #                            probability_normalize=True)
         print("TfModiscoSeqletsToPatternsFactory: seed=%d" % self.seed)
         if (self.use_louvain):
             for n_runs, level_to_return in self.louvain_num_runs_and_levels_r1:
@@ -334,8 +345,12 @@ class TfModiscoSeqletsToPatternsFactory(object):
                 n_leiden_iterations=self.n_leiden_iterations_r1,
                 verbose=self.verbose)
 
-        affmat_transformer_r2 = affinitymat.transformers.SymmetrizeByAddition(
-                                probability_normalize=True)
+        #No longer a need for symmetrization because am symmetrizing by
+        # taking the geometric mean elsewhere
+        affmat_transformer_r2 =\
+            affinitymat.transformers.AdhocAffMatTransformer(lambda x: x)
+        #affmat_transformer_r2 = affinitymat.transformers.SymmetrizeByAddition(
+        #                        probability_normalize=True)
         if (self.use_louvain):
             for n_runs, level_to_return in self.louvain_num_runs_and_levels_r2:
                 affmat_transformer_r2 = affmat_transformer_r2.chain(
@@ -383,6 +398,7 @@ class TfModiscoSeqletsToPatternsFactory(object):
                 seqlet_sort_metric=
                     lambda x: -sum([np.sum(np.abs(x[track_name].fwd)) for
                                track_name in contrib_scores_track_names]),
+            track_set=track_set, #needed for seqlet expansion
             postprocessor=postprocessor1)
 
         def sign_consistency_func(motif):
@@ -408,44 +424,6 @@ class TfModiscoSeqletsToPatternsFactory(object):
         prob_and_sim_dealbreaker_thresholds =\
             self.prob_and_pertrack_sim_dealbreaker_thresholds
 
-        spurious_merge_detector = aggregator.DetectSpuriousMerging(
-            track_names=contrib_scores_track_names,
-            track_transformer=affinitymat.core.L1Normalizer(),
-            affmat_from_1d=affinitymat.core.ContinJaccardSimilarity(
-                            make_positive=True, verbose=False),
-            diclusterer=cluster.core.LouvainCluster(
-                            level_to_return=1,
-                            max_clusters=2, contin_runs=20,
-                            verbose=False, seed=self.seed),
-            is_dissimilar_func=aggregator.PearsonCorrIsDissimilarFunc(
-                        threshold=self.threshold_for_spurious_merge_detection,
-                        verbose=self.verbose),
-            min_in_subcluster=self.final_min_cluster_size)
-
-        #similar_patterns_collapser =\
-        #    aggregator.DynamicDistanceSimilarPatternsCollapser(
-        #        pattern_to_pattern_sim_computer=
-        #            pattern_to_seqlet_sim_computer,
-        #        aff_to_dist_mat=aff_to_dist_mat,
-        #        pattern_aligner=core.CrossCorrelationPatternAligner(
-        #            pattern_comparison_settings=
-        #                affinitymat.core.PatternComparisonSettings(
-        #                    track_names=(
-        #                        contrib_scores_track_names+
-        #                        other_comparison_track_names), 
-        #                    track_transformer=
-        #                        affinitymat.MeanNormalizer().chain(
-        #                        affinitymat.MagnitudeNormalizer()), 
-        #                    min_overlap=self.min_overlap_while_sliding)),
-        #        collapse_condition=(lambda prob, aligner_sim:
-        #            any([(prob > x[0] and aligner_sim > x[1])
-        #                 for x in prob_and_sim_merge_thresholds])),
-        #        dealbreaker_condition=(lambda prob, aligner_sim:
-        #            any([(prob < x[0] and aligner_sim < x[1])              
-        #                 for x in prob_and_sim_dealbreaker_thresholds])),
-        #        postprocessor=postprocessor1,
-        #        verbose=self.verbose)
-
         similar_patterns_collapser =\
             aggregator.DynamicDistanceSimilarPatternsCollapser2(
                 pattern_comparison_settings=pattern_comparison_settings,
@@ -471,31 +449,86 @@ class TfModiscoSeqletsToPatternsFactory(object):
                 max_seqlets_subsample=self.merging_max_seqlets_subsample,
                 n_cores=self.n_cores)
 
+        subcluster_settings = {
+            "pattern_comparison_settings": pattern_comparison_settings,
+            "perplexity": self.subcluster_perplexity,
+            "n_jobs": self.n_cores,
+        }
+        spurious_merge_detector = aggregator.DetectSpuriousMerging2(
+            subcluster_settings=subcluster_settings,
+            verbose=self.verbose,
+            min_in_subcluster=max(self.final_min_cluster_size,
+                                  self.subcluster_perplexity),
+            similar_patterns_collapser=similar_patterns_collapser)
 
-        seqlet_reassigner =\
-           aggregator.ReassignSeqletsFromSmallClusters(
-            seqlet_assigner=aggregator.AssignSeqletsByBestMetric(
-                pattern_comparison_settings=pattern_comparison_settings,
-                individual_aligner_metric=
-                    core.get_best_alignment_crosscontinjaccard,
-                matrix_affinity_metric=
-                    affinitymat.core.CrossContinJaccardMultiCoreCPU(
-                        verbose=self.verbose, n_cores=self.n_cores),
-                min_similarity=self.min_similarity_for_seqlet_assignment),
-            min_cluster_size=self.final_min_cluster_size,
-            postprocessor=expand_trim_expand1,
-            verbose=self.verbose) 
+        #spurious_merge_detector = aggregator.DetectSpuriousMerging(
+        #    track_names=contrib_scores_track_names,
+        #    track_transformer=affinitymat.core.L1Normalizer(),
+        #    affmat_from_1d=affinitymat.core.ContinJaccardSimilarity(
+        #                    make_positive=True, verbose=False),
+        #    diclusterer=cluster.core.LouvainCluster(
+        #                    level_to_return=1,
+        #                    max_clusters=2, contin_runs=20,
+        #                    verbose=False, seed=self.seed),
+        #    is_dissimilar_func=aggregator.PearsonCorrIsDissimilarFunc(
+        #                threshold=self.threshold_for_spurious_merge_detection,
+        #                verbose=self.verbose),
+        #    min_in_subcluster=self.final_min_cluster_size)
+
+        #similar_patterns_collapser =\
+        #    aggregator.DynamicDistanceSimilarPatternsCollapser(
+        #        pattern_to_pattern_sim_computer=
+        #            pattern_to_seqlet_sim_computer,
+        #        aff_to_dist_mat=aff_to_dist_mat,
+        #        pattern_aligner=core.CrossCorrelationPatternAligner(
+        #            pattern_comparison_settings=
+        #                affinitymat.core.PatternComparisonSettings(
+        #                    track_names=(
+        #                        contrib_scores_track_names+
+        #                        other_comparison_track_names), 
+        #                    track_transformer=
+        #                        affinitymat.MeanNormalizer().chain(
+        #                        affinitymat.MagnitudeNormalizer()), 
+        #                    min_overlap=self.min_overlap_while_sliding)),
+        #        collapse_condition=(lambda prob, aligner_sim:
+        #            any([(prob > x[0] and aligner_sim > x[1])
+        #                 for x in prob_and_sim_merge_thresholds])),
+        #        dealbreaker_condition=(lambda prob, aligner_sim:
+        #            any([(prob < x[0] and aligner_sim < x[1])              
+        #                 for x in prob_and_sim_dealbreaker_thresholds])),
+        #        postprocessor=postprocessor1,
+        #        verbose=self.verbose)
+
+        pattern_filterer = pattern_filterer_module.MinSeqletSupportFilterer(
+            min_seqlet_support=self.final_min_cluster_size).chain(
+                pattern_filterer_module.MinICinWindow(
+                    window_size=self.min_ic_windowsize,
+                    min_ic_in_window=self.min_ic_in_window,
+                    background=bg_freq,
+                    sequence_track_name=onehot_track_name,
+                    ppm_pseudocount=self.ppm_pseudocount   
+                ) 
+            )
+
+        #seqlet_reassigner =\
+        #   aggregator.ReassignSeqletsFromSmallClusters(
+        #    seqlet_assigner=aggregator.AssignSeqletsByBestMetric(
+        #        pattern_comparison_settings=pattern_comparison_settings,
+        #        individual_aligner_metric=
+        #            core.get_best_alignment_crosscontinjaccard,
+        #        matrix_affinity_metric=
+        #            affinitymat.core.CrossContinJaccardMultiCoreCPU(
+        #                verbose=self.verbose, n_cores=self.n_cores),
+        #        min_similarity=self.min_similarity_for_seqlet_assignment,
+        #        track_set=track_set),
+        #    min_cluster_size=self.final_min_cluster_size,
+        #    postprocessor=expand_trim_expand1,
+        #    verbose=self.verbose) 
 
         final_postprocessor = aggregator.ExpandSeqletsToFillPattern(
                                         track_set=track_set,
                                         flank_to_add=self.final_flank_to_add) 
 
-        final_subcluster_settings = {
-            "pattern_comparison_settings": pattern_comparison_settings,
-            "perplexity": self.final_subcluster_perplexity,
-            "n_jobs": self.n_cores,
-        }
-
         return TfModiscoSeqletsToPatterns(
                 seqlets_sorter=seqlets_sorter,
                 initclusterer_factory=initclusterer_factory,
@@ -506,17 +539,29 @@ class TfModiscoSeqletsToPatternsFactory(object):
                 filter_mask_from_correlation=filter_mask_from_correlation,
                 filter_beyond_first_round=self.filter_beyond_first_round,
                 skip_fine_grained=self.skip_fine_grained,
-                density_adapted_affmat_transformer=
-                    density_adapted_affmat_transformer,
+                aff_to_dist_mat=aff_to_dist_mat,
+                tsne_perplexity=self.tsne_perplexity,
+                #density_adapted_affmat_transformer=
+                #    density_adapted_affmat_transformer,
                 clusterer_per_round=clusterer_per_round,
                 seqlet_aggregator=seqlet_aggregator,
                 sign_consistency_func=sign_consistency_func,
+                subcluster_settings=subcluster_settings,
                 spurious_merge_detector=spurious_merge_detector,
                 similar_patterns_collapser=similar_patterns_collapser,
-                seqlet_reassigner=seqlet_reassigner,
+                #seqlet_reassigner=seqlet_reassigner,
+                pattern_filterer=pattern_filterer,
                 final_postprocessor=final_postprocessor,
-                final_subcluster_settings=final_subcluster_settings,
-                verbose=self.verbose)
+                verbose=self.verbose,
+                n_cores=self.n_cores,
+                other_config={
+                 'onehot_track_name': onehot_track_name,
+                 'contrib_scores_track_names': contrib_scores_track_names,
+                 'hypothetical_contribs_track_names':
+                    hypothetical_contribs_track_names,
+                 'track_signs': track_signs, 
+                 'other_comparison_track_names': other_comparison_track_names},
+                )
 
     def save_hdf5(self, grp):
         grp.attrs['jsonable_config'] =\
@@ -528,15 +573,18 @@ class SeqletsToPatternsResults(object):
     def __init__(self,
                  each_round_initcluster_motifs, 
                  patterns, 
-                 patterns_withoutreassignment,
+                 remaining_patterns,
                  pattern_merge_hierarchy,
                  cluster_results,
                  total_time_taken,
-                 success=True, **kwargs):
+                 other_config={},
+                 success=True,
+                 **kwargs):
         self.each_round_initcluster_motifs = each_round_initcluster_motifs
+        self.other_config = other_config
         self.success = success
         self.patterns = patterns
-        self.patterns_withoutreassignment = patterns_withoutreassignment
+        self.remaining_patterns = remaining_patterns
         self.pattern_merge_hierarchy = pattern_merge_hierarchy
         self.cluster_results = cluster_results
         self.total_time_taken = total_time_taken
@@ -579,12 +627,12 @@ class SeqletsToPatternsResults(object):
                         track_set=track_set)
             patterns = util.load_patterns(grp=grp["patterns"],
                                           track_set=track_set) 
-            if "patterns_withoutreassignment" in grp:
-                patterns_withoutreassignment = util.load_patterns(
-                    grp=grp["patterns_withoutreassignment"],
+            if "remaining_patterns" in grp:
+                remaining_patterns = util.load_patterns(
+                    grp=grp["remaining_patterns"],
                     track_set=track_set) 
             else: #backwards compatibility
-                patterns_withoutreassignment = []
+                remaining_patterns = []
             cluster_results = None
             total_time_taken = grp.attrs["total_time_taken"]
             if ("pattern_merge_hierarchy" in grp):
@@ -597,7 +645,7 @@ class SeqletsToPatternsResults(object):
             return cls(
                 each_round_initcluster_motifs=each_round_initcluster_motifs,
                 patterns=patterns,
-                patterns_withoutreassignment=patterns_withoutreassignment,
+                remaining_patterns=remaining_patterns,
                 pattern_merge_hierarchy=pattern_merge_hierarchy,
                 cluster_results=cluster_results,
                 total_time_taken=total_time_taken)
@@ -605,11 +653,13 @@ class SeqletsToPatternsResults(object):
             return cls(success=False, patterns=None, cluster_results=None,
                        total_time_taken=None,
                        each_round_initcluster_motifs=None,
-                       patterns_withoutreassignment=None,
+                       remaining_patterns=None,
                        pattern_merge_hierarchy=None)
 
     def save_hdf5(self, grp):
         grp.attrs["success"] = self.success
+        grp.attrs["other_config"] =\
+            json.dumps(self.other_config, indent=4, separators=(',', ': ')) 
         if (self.success):
             grp.attrs["total_time_taken"] = self.total_time_taken
             if (self.each_round_initcluster_motifs is not None):
@@ -618,8 +668,8 @@ class SeqletsToPatternsResults(object):
             util.save_patterns(self.patterns,
                                grp.create_group("patterns"))
             util.save_patterns(
-                self.patterns_withoutreassignment,
-                grp.create_group("patterns_withoutreassignment"))
+                self.remaining_patterns,
+                grp.create_group("remaining_patterns"))
             self.cluster_results.save_hdf5(grp.create_group("cluster_results"))   
             grp.attrs['total_time_taken'] = self.total_time_taken
             self.pattern_merge_hierarchy.save_hdf5(
@@ -642,15 +692,20 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
                        filter_mask_from_correlation,
                        filter_beyond_first_round,
                        skip_fine_grained,
-                       density_adapted_affmat_transformer,
+                       aff_to_dist_mat,
+                       tsne_perplexity,
+                       #density_adapted_affmat_transformer,
                        clusterer_per_round,
                        seqlet_aggregator,
                        sign_consistency_func,
                        spurious_merge_detector,
                        similar_patterns_collapser,
-                       seqlet_reassigner,
+                       pattern_filterer,
+                       #seqlet_reassigner,
                        final_postprocessor,
-                       final_subcluster_settings,
+                       subcluster_settings,
+                       n_cores,
+                       other_config={},
                        verbose=True):
 
         self.seqlets_sorter = seqlets_sorter
@@ -662,19 +717,24 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
         self.filter_mask_from_correlation = filter_mask_from_correlation
         self.filter_beyond_first_round = filter_beyond_first_round
         self.skip_fine_grained = skip_fine_grained
-        self.density_adapted_affmat_transformer =\
-            density_adapted_affmat_transformer
+        self.aff_to_dist_mat = aff_to_dist_mat
+        self.tsne_perplexity = tsne_perplexity
+        #self.density_adapted_affmat_transformer =\
+        #    density_adapted_affmat_transformer
         self.clusterer_per_round = clusterer_per_round 
         self.seqlet_aggregator = seqlet_aggregator
         self.sign_consistency_func = sign_consistency_func
         
         self.spurious_merge_detector = spurious_merge_detector
         self.similar_patterns_collapser = similar_patterns_collapser
-        self.seqlet_reassigner = seqlet_reassigner
+        #self.seqlet_reassigner = seqlet_reassigner
+        self.pattern_filterer = pattern_filterer
         self.final_postprocessor = final_postprocessor
 
         self.verbose = verbose
-        self.final_subcluster_settings = final_subcluster_settings
+        self.subcluster_settings = subcluster_settings
+        self.n_cores = n_cores
+        self.other_config = other_config
 
     def get_cluster_to_aggregate_motif(self, seqlets, cluster_indices,
                                        sign_consistency_check,
@@ -709,6 +769,18 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
                         cluster_to_eliminated_motif[i] = motif
         return cluster_to_motif, cluster_to_eliminated_motif
 
+    def do_density_adaptation(self, new_rows_distmat_nn, new_rows_nn,
+                                    new_rows_betas, new_rows_normfactors):
+        new_rows_densadapted_affmat_nn = []
+        for i in range(len(new_rows_distmat_nn)):
+            densadapted_row = []
+            for j,distance in zip(new_rows_nn[i], new_rows_distmat_nn[i]):
+                densadapted_row.append(np.sqrt(
+                  (np.exp(-distance/new_rows_betas[i])/new_rows_normfactors[i])
+                 *(np.exp(-distance/new_rows_betas[j])/
+                   new_rows_normfactors[j]))) 
+            new_rows_densadapted_affmat_nn.append(densadapted_row)
+        return new_rows_densadapted_affmat_nn
 
     def __call__(self, seqlets):
 
@@ -760,7 +832,7 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
                 return SeqletsToPatternsResults(
                         each_round_initcluster_motifs=None,
                         patterns=None,
-                        patterns_withoutreassignment=None,
+                        remaining_patterns=None,
                         pattern_merge_hierarchy=None,
                         cluster_results=None, 
                         total_time_taken=None,
@@ -935,11 +1007,63 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
             #del filtered_affmat
             #density_adapted_affmats.append(density_adapted_affmat)
 
-            coo_density_adapted_affmat =\
-                self.density_adapted_affmat_transformer(
-                    filtered_affmat_nn, seqlet_neighbors)
+            #apply aff_to_dist_mat one row at a time
+            distmat_nn = [self.aff_to_dist_mat(affinity_mat=x)
+                          for x in filtered_affmat_nn] 
             del filtered_affmat_nn
 
+            if (self.verbose):
+                print("Symmetrizing nearest neighbors")
+            #Note: the fine-grained similarity metric isn't actually symmetric
+            # because a different input will get padded with zeros depending
+            # on which seqlets are specified as the filters and which seqlets
+            # are specified as the 'thing to scan'. So explicit symmetrization
+            # is worthwhile
+            sym_seqlet_neighbors, sym_distmat_nn = util.symmetrize_nn_distmat(
+                distmat_nn=distmat_nn, nn=seqlet_neighbors,
+                average_with_transpose=True)
+            del distmat_nn
+            del seqlet_neighbors
+
+            if (self.verbose):
+                print("Computing betas for density adaptation")
+
+            #Compute beta values for the density adaptation. *store it*
+            betas_and_ps = Parallel(n_jobs=self.n_cores)(
+                     delayed(util.binary_search_perplexity)(
+                          self.tsne_perplexity, distances)
+                     for distances in sym_distmat_nn)
+            betas = np.array([x[0] for x in betas_and_ps])
+
+            if (self.verbose):
+                print("Computing normalizing denominators")
+
+            #also compute the normalization factor needed to get probs to sum to 1
+            #note: sticking to lists here because different rows of
+            # sym_distmat_nn may have different lengths after adding in
+            # the symmetric pairs
+            densadapted_affmat_nn_unnorm = [np.exp(-np.array(distmat_row)/beta)
+                for distmat_row, beta in zip(sym_distmat_nn, betas)]
+            normfactors = np.array([max(np.sum(x),1e-8) for x in
+                                    densadapted_affmat_nn_unnorm])
+
+            sym_densadapted_affmat_nn = self.do_density_adaptation(
+                new_rows_distmat_nn=sym_distmat_nn,
+                new_rows_nn=sym_seqlet_neighbors,
+                new_rows_betas=betas,
+                new_rows_normfactors=normfactors)
+
+            util.verify_symmetric_nn_affmat(
+                affmat_nn=sym_densadapted_affmat_nn,
+                nn=sym_seqlet_neighbors)
+
+            #Make csr matrix
+            csr_density_adapted_affmat =\
+                util.coo_matrix_from_neighborsformat(
+                    entries=sym_densadapted_affmat_nn,
+                    neighbors=sym_seqlet_neighbors,
+                    ncols=len(sym_densadapted_affmat_nn)).tocsr()
+
             if (self.verbose):
                 print("(Round "+str(round_num)+") Computing clustering")
                 print_memory_use()
@@ -949,9 +1073,9 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
             #                            initclusters=filtered_initclusters)
             #del density_adapted_affmat
             #cluster_results_sets.append(cluster_results)
-            cluster_results = clusterer(coo_density_adapted_affmat,
+            cluster_results = clusterer(csr_density_adapted_affmat,
                                         initclusters=filtered_initclusters)
-            del coo_density_adapted_affmat
+            del csr_density_adapted_affmat
 
             num_clusters = max(cluster_results.cluster_indices+1)
             cluster_idx_counts = Counter(cluster_results.cluster_indices)
@@ -996,7 +1120,7 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
             return SeqletsToPatternsResults(
                     each_round_initcluster_motifs=None,
                     patterns=None,
-                    patterns_withoutreassignment=None,
+                    remaining_patterns=None,
                     pattern_merge_hierarchy=None,
                     cluster_results=None, 
                     total_time_taken=None,
@@ -1019,16 +1143,20 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
             sys.stdout.flush()
 
         if (self.verbose):
-            print("Performing seqlet reassignment")
+            print("Performing filtering")
             print_memory_use()
             sys.stdout.flush()
-        reassigned_patterns = self.seqlet_reassigner(merged_patterns)
-        final_patterns = self.final_postprocessor(reassigned_patterns)
-        final_patterns_withoutreassignment =\
-            self.final_postprocessor(merged_patterns)
+
+        final_patterns, remaining_patterns = self.pattern_filterer(
+                                                    merged_patterns)
+        #reassigned_patterns = self.seqlet_reassigner(merged_patterns)
+        final_patterns = self.final_postprocessor(final_patterns)
+        remaining_patterns =\
+            self.final_postprocessor(remaining_patterns)
+
         if (self.verbose):
             print("Got "+str(len(final_patterns))
-                  +" patterns after reassignment")
+                  +" patterns after filtering")
             print_memory_use()
             sys.stdout.flush()
 
@@ -1045,12 +1173,12 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
             print("On pattern",patternidx)
             pattern.compute_subclusters_and_embedding(
                 verbose=self.verbose,
-                **self.final_subcluster_settings)
+                **self.subcluster_settings)
 
         results = SeqletsToPatternsResults(
             each_round_initcluster_motifs=each_round_initcluster_motifs,             
             patterns=final_patterns,
-            patterns_withoutreassignment=final_patterns_withoutreassignment,
+            remaining_patterns=remaining_patterns,
             seqlets=filtered_seqlets, #last stage of filtered seqlets
             #affmat=filtered_affmat,
             cluster_results=cluster_results, 
@@ -1068,6 +1196,7 @@ class TfModiscoSeqletsToPatterns(AbstractSeqletsToPatterns):
 
             merged_patterns=merged_patterns,
             pattern_merge_hierarchy=pattern_merge_hierarchy,
-            reassigned_patterns=reassigned_patterns)
+            #reassigned_patterns=reassigned_patterns
+        )
 
         return results
