@@ -6,6 +6,8 @@ import numpy as np
 import h5py
 import traceback
 import scipy.sparse
+from sklearn.metrics import average_precision_score, precision_recall_curve
+from sklearn.isotonic import IsotonicRegression
 
 
 def print_memory_use():
@@ -167,6 +169,8 @@ def firstd(x_values, y_values):
     return x_midpoints, rise_over_run
 
 
+#TODO: this can prob be replaced with np.sum(
+# util.rolling_window(a=arr, window=window_size), axis=-1)  
 def cpu_sliding_window_sum(arr, window_size):
     assert len(arr) >= window_size, str(len(arr))+" "+str(window_size)
     to_return = np.zeros(len(arr)-window_size+1)
@@ -512,6 +516,7 @@ def compute_per_position_ic(ppm, background, pseudocount):
 
 #rolling_window is from this blog post by Erik Rigtorp:
 # https://rigtorp.se/2011/01/01/rolling-statistics-numpy.html
+#The last axis of a will be subject to the windowing
 def rolling_window(a, window):
     shape = a.shape[:-1] + (a.shape[-1] - window + 1, window)
     strides = a.strides + (a.strides[-1],)
@@ -523,6 +528,11 @@ def sliding_window_max(a, window):
     return np.max(rolling_windows_a, axis=-1) 
 
 
+def sliding_window_max(a, window):
+    rolling_windows_a = rolling_window(a, window)
+    return np.max(rolling_windows_a, axis=-1) 
+
+
 def compute_masked_cosine_sim(imp_scores, onehot_seq, weightmat): 
     strided_impscores = rolling_window(
         imp_scores.transpose((0,2,1)),
@@ -656,3 +666,317 @@ def show_or_savefig(plot_save_dir, filename):
         fname = (plot_save_dir+"/"+filename)
         plt.savefig(fname)
         print("saving plot to " + fname)
+
+
+def symmetrize_nn_distmat(distmat_nn, nn, average_with_transpose):
+    #Augment any distmat_nn entries with reciprocal entries that might be
+    # missing because "j" might be in the nearest-neighbors list of i, but
+    # i may not have made it into the nearest neighbors list for j, and vice
+    # versa
+
+    #in case the underlying distance metric isn't symmetric, average with
+    # transpose if available
+    if (average_with_transpose):
+       distmat_nn = sparse_average_with_transpose_if_available( 
+                        affmat_nn=distmat_nn, nn=nn)
+
+    nn_sets = [set(x) for x in nn]
+    augmented_distmat_nn = [list(x) for x in distmat_nn]
+    augmented_nn = [list(x) for x in nn]
+
+    for i in range(len(nn)):
+        #print(i)
+        for neighb,distance in zip(nn[i], distmat_nn[i]):
+            if i not in nn_sets[neighb]:
+                augmented_nn[neighb].append(i) 
+                augmented_distmat_nn[neighb].append(distance) 
+
+    verify_symmetric_nn_affmat(affmat_nn=augmented_distmat_nn,
+                               nn=augmented_nn)
+    
+    sorted_augmented_nn = []
+    sorted_augmented_distmat_nn = []
+    for augmented_nn_row, augmented_distmat_nn_row in zip(
+                                           augmented_nn, augmented_distmat_nn): 
+       augmented_nn_row = np.array(augmented_nn_row) 
+       augmented_distmat_nn_row = np.array(augmented_distmat_nn_row)
+       argsort_indices = np.argsort(augmented_distmat_nn_row) 
+       sorted_augmented_nn.append(augmented_nn_row[argsort_indices])
+       sorted_augmented_distmat_nn.append(
+            augmented_distmat_nn_row[argsort_indices])
+
+    #do a sanity check involving the nn sets. Make sure there are no duplicates
+    # and thye are reciprocal
+    nn_sets_2 = [set(x) for x in sorted_augmented_nn]
+    for i in range(len(sorted_augmented_nn)):
+        assert len(nn_sets_2[i])==len(sorted_augmented_nn[i])
+        for neighb in sorted_augmented_nn[i]:
+            assert i in nn_sets_2[neighb] 
+
+    verify_symmetric_nn_affmat(affmat_nn=sorted_augmented_distmat_nn,
+                               nn=sorted_augmented_nn)
+
+    return sorted_augmented_nn, sorted_augmented_distmat_nn
+
+
+def sparse_average_with_transpose_if_available(affmat_nn, nn):
+    coord_to_sim = dict([
+        ((i,j),sim) for i in range(len(affmat_nn))
+        for j,sim in zip(nn[i],affmat_nn[i]) ])
+    new_affmat_nn = [
+        np.array([
+            coord_to_sim[(i,j)] if (j,i) not in coord_to_sim else
+            0.5*(coord_to_sim[(i,j)] + coord_to_sim[(j,i)])
+            for j in nn[i]
+        ]) for i in range(len(affmat_nn))
+    ]
+    return new_affmat_nn
+
+
+def verify_symmetric_nn_affmat(affmat_nn, nn):
+    coord_to_sim = dict([
+        ((i,j),sim) for i in range(len(affmat_nn))
+        for j,sim in zip(nn[i],affmat_nn[i]) ])
+    for (i,j) in coord_to_sim.keys():
+        assert coord_to_sim[(i,j)]==coord_to_sim[(j,i)],\
+                (i,j,coord_to_sim[(i,j)], coord_to_sim[(j,i)])
+
+
+def subsample_pattern(pattern, num_to_subsample):
+    from . import core
+    seqlets_and_alnmts_list = list(pattern.seqlets_and_alnmts)
+    subsample = [seqlets_and_alnmts_list[i]
+                 for i in
+                 np.random.RandomState(1234).choice(
+                     a=np.arange(len(seqlets_and_alnmts_list)),
+                     replace=False,
+                     size=num_to_subsample)]
+    return core.AggregatedSeqlet(seqlets_and_alnmts_arr=subsample) 
+
+
+class ClasswisePrecisionScorer(object):
+
+    def __init__(self, true_classes, class_membership_scores):
+        #true_classes has len num_examples
+        #class_membership_scores has dims num_examples x classes
+        self.num_classes = max(true_classes)+1
+        assert len(set(true_classes))==self.num_classes
+        assert len(true_classes)==len(class_membership_scores)
+        assert class_membership_scores.shape[1] == self.num_classes
+
+        argmax_class_from_scores = np.argmax(
+            class_membership_scores, axis=-1)
+        print("Accuracy:", np.mean(true_classes==argmax_class_from_scores))
+        
+        prec_ir_list = []
+        precision_list = []
+        recall_list = []
+        thresholds_list = []
+        for classidx in range(self.num_classes):
+            class_membership_mask = true_classes==classidx
+            ir = IsotonicRegression(out_of_bounds='clip').fit(
+                X=class_membership_scores[:,classidx],
+                y=1.0*(class_membership_mask))
+            prec_ir_list.append(ir)
+            precision, recall, thresholds = precision_recall_curve(
+                    y_true=1.0*(class_membership_mask),
+                    probas_pred=class_membership_scores[:,classidx]) 
+            precision_list.append(precision)
+            recall_list.append(recall)
+            thresholds_list.append(thresholds)
+
+        self.prec_ir_list = prec_ir_list
+        self.precision_list = precision_list
+        self.recall_list = recall_list
+        self.thresholds_list = thresholds_list
+
+    def score_percentile(self, score, top_class):
+        if (hasattr(score, '__iter__')==False):
+            return 1- self.recall_list[top_class][
+                        np.searchsorted(self.thresholds_list[top_class],
+                                        score)]
+        else:
+            if (hasattr(top_class, '__iter__')==False):
+                return 1 - self.recall_list[top_class][
+                            np.searchsorted(self.thresholds_list[top_class],
+                                            score)]
+            else:
+                return 1 - np.array([self.recall_list[y][
+                            np.searchsorted(self.thresholds_list[y],x)]
+                            for x,y in zip(score, top_class)])
+
+    def __call__(self, score, top_class):
+        if (hasattr(score, '__iter__')==False):
+            return self.prec_ir_list[top_class].transform([score])[0]
+        else:
+            if (hasattr(top_class, '__iter__')==False):
+                return self.prec_ir_list[top_class].transform(score)
+            else:
+                return np.array([self.prec_ir_list[y].transform([x])[0]
+                        for x,y in zip(score, top_class)])
+
+
+def trim_patterns_by_ic(patterns, window_size,
+                        onehot_track_name, bg_freq):
+    from . import aggregator 
+    trimmer = aggregator.TrimToBestWindowByIC(
+                window_size=window_size,
+                onehot_track_name=onehot_track_name,
+                bg_freq=bg_freq)
+    return trimmer(patterns)
+
+
+def apply_subclustering_to_patterns(patterns, track_names,
+                                    n_jobs, perplexity=50, verbose=True):
+    from . import affinitymat
+    for pattern in patterns:
+       pattern.compute_subclusters_and_embedding(
+         pattern_comparison_settings=
+            affinitymat.core.PatternComparisonSettings( 
+                track_names=track_names, 
+                track_transformer=affinitymat.L1Normalizer(),
+                min_overlap=None), #min_overlap argument is irrelevant here 
+         perplexity=perplexity, n_jobs=n_jobs, verbose=verbose) 
+
+
+class ModularityScorer(object):
+
+    def __init__(self, clusters, nn, affmat_nn,
+                       cluster_to_supercluster_mapping=None):
+
+        verify_symmetric_nn_affmat(affmat_nn=affmat_nn, nn=nn)
+
+        #assert that affmat has the same len as clusters 
+        assert len(clusters)==len(affmat_nn), (len(clusters), len(affmat_nn))
+        assert np.max([np.max(x) for x in nn])==len(clusters)-1, (
+                np.max([np.max(x) for x in nn]), len(clusters))
+        self.num_clusters = max(clusters)+1
+        assert len(set(clusters))==self.num_clusters
+
+        if (cluster_to_supercluster_mapping is None):
+            cluster_to_supercluster_mapping = dict([(i,i) for i in
+                                                    range(self.num_clusters)])
+        self.cluster_to_supercluster_mapping = cluster_to_supercluster_mapping
+        self.build_supercluster_masks()
+
+        self.clusters = clusters
+        self.twom = np.sum([np.sum(x) for x in affmat_nn]) 
+        sigmatot_arr = []
+        for clusteridx in range(self.num_clusters):
+            withincluster_idxs = np.nonzero(1.0*(clusters==clusteridx))[0]
+            sigmatot_arr.append(np.sum([
+              np.sum(affmat_nn[i]) for i in withincluster_idxs]))
+        self.sigmatot_arr = np.array(sigmatot_arr)
+
+        #compute the modularity deltas 
+        self_modularity_deltas =\
+            self.get_modularity_deltas(new_rows_affmat_nn=affmat_nn,
+                                       new_rows_nn=nn)
+
+        self.precision_scorer = ClasswisePrecisionScorer(
+            true_classes=np.array([self.cluster_to_supercluster_mapping[x]
+                                   for x in self.clusters]),
+            class_membership_scores=
+                self.get_supercluster_scores(scores=self_modularity_deltas))
+
+    def build_supercluster_masks(self):
+        #build a matrix that is num_superclusters x num_clusters were
+        # the entries are booleans indicating membership of a cluster in
+        # the corresponding supercluster
+        self.num_superclusters = max(
+            self.cluster_to_supercluster_mapping.values())+1
+        withinsupercluster_masks =\
+            np.zeros((self.num_superclusters, self.num_clusters))
+        for clusteridx,superclusteridx in\
+            self.cluster_to_supercluster_mapping.items():
+            withinsupercluster_masks[superclusteridx, clusteridx] = 1
+        self.withinsupercluster_masks = (withinsupercluster_masks > 0.0) 
+
+    def get_supercluster_scores(self, scores):
+        #given a scores matrix that is num_examples x num_clusters, prepare
+        # a matrix that is num_examples x num_superclusters, where the
+        # supercluster score is derived by taking a max over the clusters
+        # belonging to the supercluster
+        supercluster_scores = []
+        for withinsupercluster_mask in self.withinsupercluster_masks:
+            supercluster_scores.append(
+                np.max(scores[:,withinsupercluster_mask], axis=-1)) 
+        return np.array(supercluster_scores).transpose() 
+        
+    def get_modularity_deltas(self, new_rows_affmat_nn, new_rows_nn):
+        #From https://en.wikipedia.org/wiki/Louvain_method#Algorithm
+        #Note that the formula for deltaQ that they have assumes the graph isn't
+        # being modified and reduces to:
+        # 2(k_in)/(2m) - 2*(Sigma_tot)*k_tot/((2m)^2)
+        #If we assume the graph is modified, this would be:
+        # 2(k_in)/(2m + k_tot) - 2*(Sigma_tot + k_in)*k_tot/((2m + k_tot)^2)
+        assert np.max([np.max(x) for x
+                       in new_rows_affmat_nn]) < len(self.clusters)
+        k_tot = np.array([np.sum(x) for x in new_rows_affmat_nn])
+        kin_arr = [] #will have dims of things_to_score X num_clusters
+        for clusteridx in range(self.num_clusters):
+            withincluster_idxs_set = set(
+                np.nonzero(1.0*(self.clusters==clusteridx))[0])
+            #this produces dims of num_clusters X things_to_score
+            # will transpose later
+            kin_arr.append(np.array([
+              np.sum([sim for (sim,nn_idx) in
+                      zip(sim_row, nn_row) if
+                      nn_idx in withincluster_idxs_set])
+              for (sim_row, nn_row) in zip(new_rows_affmat_nn, new_rows_nn)]))
+        kin_arr = np.array(kin_arr).transpose((1,0)) 
+        assert kin_arr.shape[1]==self.num_clusters
+        assert kin_arr.shape[0]==len(new_rows_affmat_nn)
+        assert k_tot.shape[0]==len(new_rows_affmat_nn)
+        assert self.sigmatot_arr.shape[0]==self.num_clusters
+        assert len(k_tot.shape)==1
+        assert len(self.sigmatot_arr.shape)==1
+        assert len(kin_arr.shape)==2
+        
+        #Let's just try with the scoring that assumes the new entries
+        # were already part of the graph and we are just computing the
+        # score for going from singleton to being part of the cluster
+        # 2(k_in)/(2m + k_tot) - 2*(Sigma_tot + k_in)*k_tot/((2m + k_tot)^2)
+        modularity_deltas = (
+            ((2*kin_arr)/(self.twom + k_tot[:,None]))
+            - ((2*(self.sigmatot_arr[None,:] + kin_arr)*k_tot[:,None])/
+               np.square(self.twom + k_tot[:,None])))
+         
+        return modularity_deltas
+
+    #new_rows_affmat_nn and new_rows_nn should be [things_to_score X num_nn],
+    # where nn is in the space of the original nodes used to define the clusters
+    #new_rows_affmat_nn contains the sims to the nearest neighbors,
+    # new_rows_nn contains the nearest neighbor indices 
+    def __call__(self, new_rows_affmat_nn, new_rows_nn,
+                       hits_to_return_per_input):
+        modularity_deltas = self.get_supercluster_scores(
+                                   scores=self.get_modularity_deltas(
+                                       new_rows_affmat_nn=new_rows_affmat_nn,
+                                       new_rows_nn=new_rows_nn))
+
+        assert hits_to_return_per_input >= 1
+        #get the top hits_to_return_per_input matches
+        sorted_class_matches = np.argsort(-modularity_deltas, axis=-1)[:,
+                                              0:hits_to_return_per_input]
+        sorted_class_match_scores = modularity_deltas[
+            np.arange(len(sorted_class_matches))[:,None],
+            sorted_class_matches]
+
+        precisions = self.precision_scorer(
+           score=sorted_class_match_scores.ravel(),
+           top_class=sorted_class_matches.ravel()).reshape(
+                sorted_class_matches.shape)
+
+        percentiles = self.precision_scorer.score_percentile(
+           score=sorted_class_match_scores.ravel(),
+           top_class=sorted_class_matches.ravel()).reshape(
+                sorted_class_matches.shape)
+
+        #argmax_classes = np.argmax(modularity_deltas, axis=-1)
+        #argmax_class_scores = modularity_deltas[
+        #     np.arange(len(argmax_classes)),argmax_classes] 
+        return (sorted_class_matches, percentiles, precisions,
+                sorted_class_match_scores)
+
+
